@@ -2,6 +2,7 @@
 
 import sys
 import logging
+import time
 
 from pymycobot.log import setup_logging
 from pymycobot.error import calibration_parameters
@@ -174,6 +175,7 @@ class MyCobotCommandGenerator(DataProcessor):
         return self._mesg(ProtocolCode.IS_POWER_ON, has_reply=True)
 
     def release_all_servos(self):
+        """Relax all joints"""
         return self._mesg(ProtocolCode.RELEASE_ALL_SERVOS)
 
     def is_controller_connected(self):
@@ -181,7 +183,15 @@ class MyCobotCommandGenerator(DataProcessor):
         return self._mesg(ProtocolCode.IS_CONTROLLER_CONNECTED, has_reply=True)
 
     def read_next_error(self):
-        """Robot Error Detection"""
+        """Robot Error Detection
+        
+        Return:
+            list len 7.
+            0 : No abnormality
+            1 : Communication disconnected
+            2 : Unstable communication
+            3 : Servo abnormality
+        """
         return self._mesg(ProtocolCode.READ_NEXT_ERROR, has_reply=True)    
     
     def set_free_mode(self, flag):
@@ -214,9 +224,10 @@ class MyCobotCommandGenerator(DataProcessor):
 
         Args:
             id : Joint id(genre.Angle)\n
-                    For mycobot: int 1-6.\n
-                    For mypalletizer: int 1-4.
-                    For mypalletizer 340: int 1-3.
+                    for mycobot: int 1-6.\n
+                    for mypalletizer: int 1-4.
+                    for mypalletizer 340: int 1-3.
+                    for myArm: int 1 - 7.
             degree : degree value(float)(about -170 ~ 170).
             speed : (int) 0 ~ 100
         """
@@ -232,7 +243,8 @@ class MyCobotCommandGenerator(DataProcessor):
                         for mycobot: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0].\n
                         for mypalletizer: [0.0, 0.0, 0.0, 0.0]
                         for mypalletizer 340: [0.0, 0.0, 0.0]
-            speed : (int) 0 ~ 100
+                        for myArm: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0].\n
+            speed : (int) 1 ~ 100
         """
         # self.calibration_parameters(degrees=degrees, speed=speed)
         degrees = [self._angle2int(degree) for degree in degrees]
@@ -254,9 +266,9 @@ class MyCobotCommandGenerator(DataProcessor):
 
         Args:
             id(int) : coord id(genre.Coord)\n
-                        For mycobot: int 1-6.\n
-                        For mypalletizer: int 1-4.
-                        For mypalletizer 340: int 1-3.
+                        for mycobot: int 1-6.\n
+                        for mypalletizer: int 1-4.
+                        for mypalletizer 340: int 1-3.
             coord(float) : coord value, mm
             speed(int) : 0 ~ 100
         """
@@ -332,9 +344,10 @@ class MyCobotCommandGenerator(DataProcessor):
 
         Args:
             joint_id: int
-                    For mycobot: int 1-6.\n
-                    For mypalletizer: int 1-4.
-                    For mypalletizer 340: int 1-3.
+                    for mycobot: int 1-6.\n
+                    for mypalletizer: int 1-4.
+                    for mypalletizer 340: int 1-3.
+                    for myArm: int 1 - 7.
             direction: 0 - decrease, 1 - increase
             speed: int (0 - 100)
         """
@@ -345,9 +358,9 @@ class MyCobotCommandGenerator(DataProcessor):
 
         Args:
             coord_id: int
-                    For mycobot: int 1-6.\n
-                    For mypalletizer: int 1-4.
-                    For mypalletizer 340: int 1-3.
+                    for mycobot: int 1-6.\n
+                    for mypalletizer: int 1-4.
+                    for mypalletizer 340: int 1-3.
             direction: 0 - decrease, 1 - increase
             speed: int (0 - 100)
         """
@@ -358,8 +371,9 @@ class MyCobotCommandGenerator(DataProcessor):
 
         Args:
             joint_id: int
-                    For mycobot: int 1-6.\n
-                    For mypalletizer: int 1-4.
+                    for mycobot: int 1-6.\n
+                    for mypalletizer: int 1-4.
+                    for myArm: int 1 - 7.
             angle: -180 ~ 180
             speed: int (0 - 100)
         """
@@ -369,7 +383,10 @@ class MyCobotCommandGenerator(DataProcessor):
         """step mode
 
         Args:
-            joint_id: int 1-6.
+            joint_id:
+                for mycobot: int 1-6.
+                for mypalletizer: int 1-4.
+                for myArm: int 1 - 7.
             increment: 
             speed: int (0 - 100)
         """
@@ -409,6 +426,7 @@ class MyCobotCommandGenerator(DataProcessor):
                 for mycobot: Joint id 1 - 6
                 for mypalletizer: Joint id 1 - 4
                 for mycobot gripper: Joint id 7
+                for myArm: int 1 - 7.
             encoder: The value of the set encoder.
         """
         return self._mesg(ProtocolCode.SET_ENCODER, joint_id, [encoder])
@@ -417,7 +435,7 @@ class MyCobotCommandGenerator(DataProcessor):
         """Obtain the specified joint potential value. (mypalletizer 340 does not have this interface)
 
         Args:
-            joint_id: (int) 1 ~ 6
+            joint_id: (int) 1 ~ 7
 
         Returns:
             encoder: 0 ~ 4096
@@ -428,8 +446,8 @@ class MyCobotCommandGenerator(DataProcessor):
         """Set the six joints of the manipulator to execute synchronously to the specified position. (mypalletizer 340 does not have this interface)
 
         Args:
-            encoders: A encoder list, length 6.
-            sp: speed 0 ~ 100
+            encoders: A encoder list.
+            sp: speed 1 ~ 100
         """
         return self._mesg(ProtocolCode.SET_ENCODERS, encoders, sp)
 
@@ -482,7 +500,11 @@ class MyCobotCommandGenerator(DataProcessor):
         """Gets the minimum movement angle of the specified joint
 
         Args: 
-            joint_id: (int)
+            joint_id:
+                for mycobot: Joint id 1 - 6
+                for mypalletizer: Joint id 1 - 4
+                for mycobot gripper: Joint id 7
+                for myArm: int 1 - 7.
 
         Return:
             angle value(float)
@@ -494,7 +516,11 @@ class MyCobotCommandGenerator(DataProcessor):
         """Gets the maximum movement angle of the specified joint
         
         Args:
-            joint_id: (int)
+            joint_id:
+                for mycobot: Joint id 1 - 6
+                for mypalletizer: Joint id 1 - 4
+                for mycobot gripper: Joint id 7
+                for myArm: int 1 - 7.
 
         Return:
             angle value(float)
@@ -504,10 +530,14 @@ class MyCobotCommandGenerator(DataProcessor):
 
     # Servo control
     def is_servo_enable(self, servo_id):
-        """Determine whether all steering gears are connected
+        """To detect the connection state of a single joint
 
         Args:
-            servo_id: (int) 1 ~ 6
+            servo_id:
+                for mycobot: Joint id 1 - 6
+                for mypalletizer: Joint id 1 - 4
+                for mycobot gripper: Joint id 7
+                for myArm: int 1 - 7.
 
         Return:
             0 - disable
@@ -517,7 +547,7 @@ class MyCobotCommandGenerator(DataProcessor):
         return self._mesg(ProtocolCode.IS_SERVO_ENABLE, servo_id, has_reply=True)
 
     def is_all_servo_enable(self):
-        """Determine whether the specified steering gear is connected
+        """Detect the connection status of all joints
 
         Return:
             0 - disable
@@ -530,7 +560,7 @@ class MyCobotCommandGenerator(DataProcessor):
         """Set the data parameters of the specified address of the steering gear
 
         Args:
-            servo_id: Serial number of articulated steering gear, 1 - 6.
+            servo_id: Serial number of articulated steering gear, 1 - 7.
             data_id: Data address.
             value: 0 - 4096
         """
@@ -624,10 +654,8 @@ class MyCobotCommandGenerator(DataProcessor):
         """singal value"""
         return self._mesg(ProtocolCode.GET_DIGITAL_INPUT, pin_no, has_reply=True)
 
-    """
-    def set_pwm_mode(self, pin_no, channel):
-        self._mesg(Command.SET_PWM_MODE, pin_no, channel)
-    """
+    def set_pwm_mode(self, mode):
+        return self._mesg(ProtocolCode.SET_PWM_MODE, mode)
 
     def set_pwm_output(self, channel, frequency, pin_val):
         """ PWM control 
@@ -706,7 +734,8 @@ class MyCobotCommandGenerator(DataProcessor):
             password: (str) new wifi password.
         """
         self._mesg(ProtocolCode.SET_SSID_PWD) # 先发指令，再发设置的账号密码
-        return self._mesg(ProtocolCode.SET_SSID_PWD, account, password)
+        time.sleep(0.02)
+        return self._mesg(ProtocolCode.SET_SSID_PWD, account, password, has_reply=True)
 
     def get_ssid_pwd(self):
         """Get connected wifi account and password. (Apply to m5 or seeed)
@@ -899,9 +928,10 @@ class MyCobotCommandGenerator(DataProcessor):
         
         Args:
             id: int.
-                For mycobot: int 1-6.\n
-                For mypalletizer: int 1-4.
-                For mypalletizer 340: int 1-3.
+                for mycobot: Joint id 1 - 6
+                for mypalletizer: Joint id 1 - 4
+                for mycobot gripper: Joint id 7
+                for myArm: int 1 - 7.
             angle: 0 ~ 180 
         """
         return self._mesg(ProtocolCode.SET_JOINT_MAX, id, angle)
@@ -911,9 +941,10 @@ class MyCobotCommandGenerator(DataProcessor):
         
         Args:
             id: int.
-                For mycobot: int 1-6.\n
-                For mypalletizer: int 1-4.\n
-                For mypalletizer 340: int 1-3.
+                for mycobot: Joint id 1 - 6
+                for mypalletizer: Joint id 1 - 4
+                for mycobot gripper: Joint id 7
+                for myArm: int 1 - 7.
             angle: 0 ~ 180 
         """
         return self._mesg(ProtocolCode.SET_JOINT_MIN, id, angle)
@@ -934,7 +965,7 @@ class MyCobotCommandGenerator(DataProcessor):
         """Send all encoders and speeds
         
         Args:
-            encoders: encoders list
+            encoders: encoders list.
             speeds: Obtained by the get_servo_speeds() method 
         """
         return self._mesg(ProtocolCode.SET_ENCODERS_DRAG, encoders, speeds)
@@ -952,3 +983,29 @@ class MyCobotCommandGenerator(DataProcessor):
     def get_fresh_mode(self):
         """Query sports mode"""
         return self._mesg(ProtocolCode.GET_FRESH_MODE, has_reply = True)
+    
+    def set_gripper_mode(self, mode):
+        """Set gripper mode
+        
+        Args:
+            mode: 0 - transparent transmission. 1 - Port Mode.
+        
+        """
+        return self._mesg(ProtocolCode.SET_GRIPPER_MODE, mode)
+    
+    def get_gripper_mode(self):
+        """Get gripper mode
+        
+        Return:
+            mode: 0 - transparent transmission. 1 - Port Mode.
+        """
+        return self._mesg(ProtocolCode.GET_GRIPPER_MODE, has_reply = True)
+    
+    def get_servo_last_pdi(self, id):
+        """Obtain the pdi of a single steering gear before modification
+        
+        Args:
+            id: 1 - 6
+        """
+        return self._mesg(ProtocolCode.GET_SERVO_LASTPDI, id, has_reply = True)
+        
